C*DECK STAR 
      PROGRAM STAR

*+

*  Program:
*     STERNE

*  Purpose:
*     Calculate LTE models with line-blanketing for homogeneous
*     and inhomogeneous stellar atmospheres.

*  Version:
*     2.2       C.S.Jeffery, St Andrews   27-JUN-1995

*  History:
*     2.0       U.Heber & R.Moeller, Kiel       1990
*     1.0       B.Wolf & D.Schoenberner, Berlin 1975

*  Description:
*     This is the model atmosphere program originally written
*     model RCrB and EHe star atmospheres. Version 1 was a simple LTE
*     code written by Wolf & Schoenberner (Berlin) which accounted for
*     continuous opacity sources only, but which did include extra data
*     for carbon and nitrogen. 
*     It was given to CCP7 and implemented on the Cray XMP and VAX/VMS 
*     in about 1987. 
*
*     Subsequently Heber (Kiel) adapted the code to include He-line blocking 
*     and some UV line-blanketing, and to allow some composition stratification. 
*     Moeller (Kiel) computed opacity distribution
*     functions for He-rich mixtures, and these were incorporated in Version 2 of the code.
*     This was given to CCP7 in 1990, and implemented on the Cray XMP at RAL.
*
*     This version (2.1) of the code is an attempt to port to a Unix Sun SPARCstation. The
*     Cray code did not port easily, with very old Fortran giving problems.
*     Since the opacity library had already been ported as part of the QUB code SPECTRUM
*     the opacity sections were stripped out, and a very slightly modified form of the
*     the QUB library substituted - the library now known as OPK. A lot of work was left
*     to do integrating this library with the remaining STERNE code, and cleaning out other
*     dead code. The philosophy at this stage was to change as little as possible,
*     and only to fix code that was giving wrong answers. Hence a lot of improvements are
*     still possible.
*
*     On the port to SOLARIS, version 2.1 broke down completely. It was not clear what
*     had happened - whether it was a compiler/arithmetic problem or some violation of
*     memory. Substantial sections of code were cleaned up looking for the bug, although
*     much remains to be done. The problem was traced to subroutine ALKOEF which was
*     giving bad values to array C when scattering dominated.
*     ALKOEF code was replace by code from PARABEL in the SPECTRUM package - I had already
*     solved the problem of poorly behaved transfer solutions in scattering layers. There
*     were sufficient changes elsewhere to refeer to this as version 2.2.


*  Main features:
*     Opacities:
*        Bound-free           -  mostly from Kurucz (ATLAS)
*        Bound-free           -  carbon and nitrogen from Peach (1970)
*        bound-bound          -  ATLAS6-type opacity distribution functions
*                                (from Kurucz & Peytremann linelists)
*     Convection              -  NOT TREATED
*     Radiative transfer      -  Avrett-Loeser
*     Temperature Correction  -  Lucy-Unsold


*  To be done:
*     Opacities:
*        EITHER Kurucz ATLAS9 - type odfs to be introduced
*        OR     monochromatic opacities from Opacity Project to be used
*     Radiative Transfer:
*        EITHER a Feautrier scheme is available to replace Avrett-Loeser
*        OR     introduce a complete linearisation scheme
*     Code structure:
*        A complete rewrite is needed (with proper design methods) in order
*        to obtain a code which can be properly extended.


*  Input and Output data:
*     Unit 1:  DAT3  - Peach opacity tabels (binary) (PEACH)
*     Unit 2:  SCMOD - scratch model output (binary)
*     Unit 3:  SCRATCH
*     Unit 4:  MODIN - input model (formated or binary)
*     Unit 5:  stdin - model parameters, switches, composition and stratification
*     Unit 7:  WAVE  - wavelengths and atomic data (INDAT)
*     Unit 9:  MODEL - final model 
*     Unit 10: ODF   - Opacity distribution function (binary) (LINOP)

*-
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION ALPHAX(7,50),ZET(50),TAUX(50),FNUE(2600),BLK(2600),
     :   ALQ(50),TA(50),T(50),TNEU(50),DELT(50),PP(50),PPE(50),         
     :   DENS(50),DIST(50),DEL(50),FLX(50),GRAD(50),TNS(50),CAP4(50),   
     :   SIG(50),CAPP(50),CAPJ(50),CAPF(50),FR(15,50),
     :   DTALT(50),DELB1(50),DELB2(50),B(50)
      DIMENSION WL(3035)
      CHARACTER*80 OPAKEY

      INCLUDE 'COMMON_KIEL'

      COMMON /MODEL/    TEF,G,ALPHAX,WM    
      COMMON /PARAM/    LA,LL,NN,KK       
      COMMON /OPTDEP/   ZET,TAUX,P,Q,ALQ             
      COMMON /STN_TAUN/ TA,Z             
      COMMON /DRUCK/    PP,PPE,FR         
      COMMON /ABSKO/    CAP4,SIG          
      COMMON /INTGRA/   FLX,GRAD,TNS,CAPP,CAPJ,CAPF,DELB1,DELB2,B         
      COMMON /KONVK/    DENS,V(50),FCONV(50),DIST       
      COMMON /ALPMET/   ALPMET(7)        
      COMMON /FINT/     F1239,WLSWP,FSWP,F3121
      COMMON /SYNTHE/   WLLA,WLLL,IFI
      COMMON /QSTATIS/  FRST(5),ERST(5),KIMAX,KST
      COMMON /SCHICH/   TAUX0,IEL
      COMMON /ITEMP/    ITEMP

*.

*  Announce the program and version.
      WRITE(6,1310)

      ITEMP = 0
      DO I=1,50
        DEL(I) = 0.0
        DELT(I) = 0.0
      ENDDO
      LMAX = 2600
      NN = 50
      KK = 7     
      ZT = 0.0   
*  Convergence criterion for max(df/f)           
      EPS = 0.001

*  Input.

*  Read model parameters and switches
999   READ(5,*) TEF,GLG,ALPC,ALPN,(ALPMET(I),I=1,7)          
      READ(5,*) (ALPHAX(I,50), I = 1,KK)          
      READ(5,*) LM,NZS,IFI,IFO,IPU,IBL,IFL,KIMAX

C***  ISW2 = 1 START MIT BINAEREM INPUT (INBIN)   
C***  ISW2 = 0 START MIT LINEARER TEMPERATURSCHICHTUNG       
      READ(5,*) ISW1,ISW2,IOS,INTST,EBMV,IEL,TAUX0,ALPHAX(IEL,1)

C**** OPAKEY = 'CONTINUUM': CONTINUUM ONLY USING SUBROUTINE FLXODF
C**** OPAKEY = 'ODF': OPACITY DISTRIBUTION FUCNTION USING FLXODF
      READ(5,*) OPAKEY

      IF (IOS.NE.0) WRITE(6,1350)
      IF(LM.EQ.1.OR.ISW1.EQ.1) ISTART=1


*  Read previous model - 
      IF (LM .NE. 1) THEN           


*  - formatted (converged or unconverged).
      IF (ISW1 .EQ. 1) THEN          
        N = 4
        OPEN(N,FILE='MODIN',STATUS='OLD',FORM='FORMATTED')
*       READ(N,1003) KDATE,KTIME
        READ(N,1001) XX,YY,AL1,AL2,AL3,AL4,AL5,AL6,AL7
        READ(N,1040) X1,X2,X3,X4,X5,X6,X7        
        READ(N,1100) (T(J), J = 1,NN)    
        READ(N,*)    (GRAD(J), J = 1,NN)    
        READ(N,1120) (CAP4(J), J = 1,NN)            
        READ(N,1120) (PP(J), J = 1,NN) 
        READ(N,1120) (PPE(J), J = 1,NN)             
        READ(N,*)    (TAUX(J),J=1,NN)
        READ(N,1121) LIT
        CLOSE(N)
        LM = LIT 

        IF (TEF.EQ.0.0) TEF = XX
        IF (GLG.EQ.0.0) GLG = YY
        IF (ALPHAX(1,50).EQ.0.0) ALPHAX(1,50)=X1
        IF (ALPHAX(2,50).EQ.0.0) ALPHAX(2,50)=X2
        IF (ALPHAX(6,50).EQ.0.0) ALPHAX(6,50)=X6
        IF (ALPHAX(7,50).EQ.0.0) ALPHAX(7,50)=X7
        DO  J = 1,NN      
          PPE(J) = 10.**PPE(J)             
        ENDDO
        DELTA = TEF - XX      
        IF (ABS(DELTA) .GE. 100.) THEN          
          DO  J = 1,NN      
            T(J) = T(J) + DELTA            
         ENDDO
        ENDIF


*  - binary.
      ELSE 
        OPEN(4,FILE='MODIN',STATUS='OLD',FORM='UNFORMATTED')
        READ(4) LM            
        READ(4) (T(J), J = 1,50)         
        DO J = 50,1,-1       
          IF (T(J) .EQ. 0.) NN=J-1       
        ENDDO         
        READ(4) (GRAD(J),J = 1,NN)       
        READ(4) (CAP4(J), J = 1,NN)      
        READ(4) (DTALT(J),J=1,NN)        
        READ(4) (PPE(J),J=1,NN)         
        CLOSE(4)
      ENDIF

      ENDIF


*  Set maximum iteration counter.
      IF (KIMAX.GT.30) KIMAX=20
      IF (KIMAX.NE.0)  GOTO 1
      KIMAX=20
      IF (LM.GT.2)     KIMAX = KIMAX + LM
      IF (KIMAX.GT.20) KIMAX=20
1     CONTINUE

*  Set up composition data and wavelength limits.
      CALL INALPHA(GLG,ALPC,ALPN)
      KSC = 1               
      KA1 = 2               
      KA2 = 2               
      LMEND = LM+NZS-1      
      IF (IFI.NE.0) THEN
        READ(5,*) WLLA,WLLL
      ENDIF

*  Read wavelength data.
      CALL INDAT(WL,L3,WLSWP)
      CALL BAND(WL,LA,LL,L3,LMAX)

      DO I=1,LMAX
        BLK(I) = 0.0
        FNUE(I) = 0.0         
      ENDDO
      CALL STAND(GLG,ZT,T,FF,NN,KK,LM)            

*  Define h/he-stratification.
      CALL STRATI(IEL,TAUX0)

*  Read in peach opacity tables.
      CALL PEACH            

*  Load up common blocks for opacity routines
      TEFF = TEF
      GRAV = G
      NDEPTH = NN

      ALPHA(1) = ALPHAX(1,50)
      ALPHA(2) = ALPHAX(2,50)
      ALPHA(6) = ALPHAX(6,50)
      ALPHA(7) = ALPHAX(7,50)
      ALPHA(8) = ALPMET(1)
      ALPHA(12) = ALPMET(2)
      ALPHA(13) = ALPMET(3)
      ALPHA(14) = ALPMET(4)
      ALPHA(16) = ALPMET(5)
      ALPHA(20) = ALPMET(6)
      ALPHA(26) = ALPMET(7)

      AMU = AH + AHE* 4    + AC *12.01 + AN*14.01 
     :         + AO *16.0  + AMG*24.31 + AAL*26.98 + ASI*28.09 
     :         + AS *32.06 + ACA*40.08 + AFE*55.85

      DO J = 1,NN
         TAU(J) = TAUX(J)
      ENDDO


**  Iteration starts. **


80    CONTINUE
      REWIND 2              
      REWIND 3              
      ITEMP = ITEMP+1
*     ITEMP tells subroutine linop that it has a new temperature
      KST=0
      DO I=1,5
         ERST(I) = 0.
         FRST(I) = 0.
      ENDDO

*  Copy data to opacity commmon blocks
      DO J = 1,NN
         TEMP(J) = T(J)
      ENDDO
      WRITE (*,*) 'ATFILL'
      CALL ATFILL

*  Hydrostatic equilibrium.
      WRITE (*,*) 'HYDRO'
      CALL HYDRO(T,LM,CAP4,SIG,PP,PPE,FR,GRAD,DENS,DIST,ISW3)           
      IF (NN .EQ. 50) GOTO 103         
      NN1 = NN+1            
      DO 102 K = NN1,50     
102   T(K) = 0.             
103   CONTINUE              

*  Copy data to opacity common blocks
      DO J = 1,NN
         PG(J) = PP(J)
         PE(J) = PPE(J)
      ENDDO
      WRITE (*,*) 'IONFRN'
      CALL IONFRN

*  Calculate emergent flux.
      WRITE (6,*) 'FLXODF'
      CALL FLXODF(KSC,KA1,KA2,OPAKEY,T,WL,L3,FNUE,BLK,LL,
     :   KMAX)
      IF (IFI.NE.0) GOTO 270

*  Temperature correction.
      WRITE (*,*) 'TEMKOR'
      CALL TEMKOR(FLX,TNS,CAPP,CAPJ,CAPF,CAP4,T,TNEU,DELB1,DELB2,B,LM)  

C  Check flux constancy.
      DO J = 1,NN       
         DEL(J) = (FLX(J)-FF)/FLX(J)      
         DELT(J) = TNEU(J) - T(J)         
         IF (IOS.NE.0 .AND. ISTART.NE.1) THEN
            IF ( DTALT(J).NE.0.0 .AND. 
     :           DELT(J)/DTALT(J).GT.0. .AND. 
     :           ABS(DELT(J)).GT..5*ABS(DTALT(J)) ) THEN
               DELT(J) = (DELT(J)+DTALT(J))/2.         
               TNEU(J) = T(J) + DELT(J)     
            ENDIF
         ENDIF
      ENDDO              
      ISTART=0

*  Convergence test.
      ERR = 0.
      DO J = 1,NN       
        ADEL = ABS(DEL(J))    
        ERR = MAX(ADEL,ERR)            
      ENDDO              
      IF ( ERR.LE.EPS ) GOTO 270   
      IF ( LM.EQ.IFO )  GOTO 270       

*  Output current model.
      CALL OUTSTAT
      CALL OUT1(T,DELT,DENS,DIST,DEL,6,LM,1,TNEU,WL,FNUE,LL,IFL)
      DO J = 1,NN       
        T(J) = TNEU(J)        
        DTALT(J) = DELT(J)    
      ENDDO              
      IF ( LM.GE.LMEND ) GOTO 270
      KA1 = KA2             
      IF (KA2 .NE. 2) THEN         
        KA2 = 2               
      ELSE
        KA2 = 3               
      ENDIF
      KSC = 2               
      LM = LM+1             
  
***  Next iteration  ***      
      GOTO 80

             
 270  CONTINUE              
      CALL OUTLINE
      CALL OUT1(T,DELT,DENS,DIST,DEL,6,LM,0,TNEU,WL,FNUE,LL,IFL)
      REWIND 9              
      DO J = 1,NN       
         PPE(J) = LOG10(PPE(J))        
         PP(J)  = LOG10(PP(J))            
      ENDDO


*  Output of final model.
      OPEN(9,FILE='MODEL',STATUS='UNKNOWN',FORM='FORMATTED')
*      CALL DATE(KDATE)
*      CALL CLOCK(KTIME)
*      WRITE(6,1004) KDATE,KTIME
*      WRITE(9,1003) KDATE,KTIME
      WRITE(9,1001) TEF,GLG,(ALPMET(I),I=1,7)
      WRITE(9,1040) (ALPHAX(I,50), I = 1,KK)          
      WRITE(9,1100) (T(J), J = 1,NN)    
      WRITE(9,1301) (GRAD(J), J = 1,NN)            
      WRITE(9,1120) (CAP4(J), J = 1,NN)            
      WRITE(9,1120) (PP(J), J = 1,NN)              
      WRITE(9,1120) (PPE(J), J = 1,NN)             
      WRITE(9,1122) (TAUX(J),J=1,NN)
      WRITE(9,1121) LM
      WRITE(9,'(7G11.4)')((ALPHAX(I,J),I=1,7),J=1,NN)
      LLA=LL-LA+1
      WRITE(9,1002) LLA
      WRITE(9,'(F8.1,E12.5,F6.3)') (WL(J),FNUE(J),BLK(J),J=LA,LL)
      CLOSE (9)
      STOP 


***  Format statements ***

1000  FORMAT(F6.0,F5.2,15X,9F4.1)
1001  FORMAT(F6.0,F5.2,7E9.3)
1002  FORMAT(I4)
1003  FORMAT(2X,A8,2X,A8)
1004  FORMAT(10X,'MODEL WRITTEN TO TAPE AT DATE ',A8,2X,'TIME=',A8)
1010  FORMAT(8I4)
1030  FORMAT(4I3,F5.2,I1,2F6.2)
1350  FORMAT(3X,'OSZILLATIONSUNTERDRUECKUNG EINGESCHALTET')
1040  FORMAT(7(1E11.3))        
1100  FORMAT(10F8.1)        
1120  FORMAT(10F8.4)        
1121  FORMAT(I3)
1122  FORMAT(5E15.7)
1130  FORMAT(4(F8.1,E12.5))
1300  FORMAT(10F8.1)
1301  FORMAT(10E10.3)
1310  FORMAT(1H1,///,27X,'========',
     : ' STERNE - LTE line-blanketed stellar atmosphere program ',
     : '  ========'/27X, '        ', 
     : ' ====================================================== '/
     : /35x,
     : ' VERSION: 2.2                                27.06.1995 '/
     : 35X,
     : ' ====================================================== '/
     : )
1340  FORMAT(2F8.1)

      END        
